import { __awaiter } from 'tslib';
import { GlobalService } from 'totvs-log-mobile-foundation';
import { BaseMessage, CriticalError } from 'totvs-log-base-foundation';
import { Platform } from '@ionic/angular';
import { Injectable, NgModule } from '@angular/core';
import { Geolocation } from '@ionic-native/geolocation/ngx';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// TODO - Verificar se é válido ter classes abstratas para os utilitários
/**
 * Utilitário para auxiliar o desenvolvimento utilizando o plugin de geolocalização
 * com métodos que facilitam a solicitação de permissão e identificação de geolocalização
 */
class GeolocationUtilProvider {
    /**
     * @ignore
     * Método responsável por construir o serviço
     * platform Serviço relacionado a plataforma
     * geolocation Serviço do plugin nativo de geolocalização
     * global Serviço global da aplicação
     * @param {?} platform
     * @param {?} geolocation
     * @param {?} global
     */
    constructor(platform, geolocation, global) {
        this.geolocation = geolocation;
        this.global = global;
        /**
         * @ignore
         * Indica se a plataforma é cordova
         */
        this.isCordova = false;
        platform.ready().then((/**
         * @return {?}
         */
        () => this.isCordova = platform.is('cordova')));
    }
    /**
     * Método responsável por obter a posição corrente do usuário
     * {GeolocationOptions} options Opções para recuperação da geolocalização
     * Posição
     * @param {?} options
     * @return {?}
     */
    getCurrentPosition(options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isCordova === false) {
                /** @type {?} */
                const msg = new BaseMessage({
                    literal: 'recursoNaoEstaDisponivelMsg'
                });
                this.global.msg.showMessage(msg);
                throw new CriticalError('Platform is not cordova');
            }
            return yield this.geolocation.getCurrentPosition(options);
        });
    }
    /**
     * Método responsável por calcular a distância entre dois pontos através de suas latitudes e longitudes
     * point1 Coordenadas do primeiro ponto
     * point2 Coordenadas do segundo ponto
     * @param {?} point1
     * @param {?} point2
     * @return {?}
     */
    calculatePointsKMDistance(point1, point2) {
        if (!this.validGeoPositionPoint(point1) || !this.validGeoPositionPoint(point2)) {
            throw new Error('ERROR - Invalid values');
        }
        if (point1.latitude === point2.latitude && point1.longitude === point2.longitude) {
            return 0;
        }
        /** @type {?} */
        const radlat1 = this.degreesToRad(point1.latitude);
        /** @type {?} */
        const radlat2 = this.degreesToRad(point2.latitude);
        /** @type {?} */
        const theta = point1.longitude - point2.longitude;
        /** @type {?} */
        const radtheta = this.degreesToRad(theta);
        /** @type {?} */
        let dist = Math.sin(radlat1) * Math.sin(radlat2) + Math.cos(radlat1) * Math.cos(radlat2) * Math.cos(radtheta);
        if (dist > 1) {
            dist = 1;
        }
        dist = Math.acos(dist);
        dist = dist * 180 / Math.PI;
        dist = dist * 60 * 1.1515;
        // KM
        return dist * 1.609344;
    }
    /**
     * Converte os graus decimais em radianos
     * degrees Graus decimais
     * @param {?} degrees
     * @return {?}
     */
    degreesToRad(degrees) {
        return Math.PI * degrees / 180;
    }
    /**
     * Valida os valores do ponto recebido e retorna o resultado
     * point Ponto contendo latitude e longitude
     * @param {?} point
     * @return {?}
     */
    validGeoPositionPoint(point) {
        if (!point) {
            return false;
        }
        if (isNaN(point.latitude) || isNaN(point.longitude)) {
            return false;
        }
        if (point.latitude === null || point.longitude === null) {
            return false;
        }
        if (point.latitude > 90 || point.latitude < -90) {
            return false;
        }
        if (point.longitude > 180 || point.longitude < -180) {
            return false;
        }
        return true;
    }
}
GeolocationUtilProvider.decorators = [
    { type: Injectable }
];
/** @nocollapse */
GeolocationUtilProvider.ctorParameters = () => [
    { type: Platform },
    { type: Geolocation },
    { type: GlobalService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que contém um provedor utilitário que auxilia na
 * utilização do plugin nativo de geolocalização
 */
class GeolocationUtilModule {
}
GeolocationUtilModule.decorators = [
    { type: NgModule, args: [{
                providers: [
                    Geolocation,
                    GeolocationUtilProvider
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { GeolocationUtilModule, GeolocationUtilProvider };

//# sourceMappingURL=totvs-log-mobile-foundation-src-geolocation-util.js.map