import { __awaiter } from 'tslib';
import { UtilsFn } from 'totvs-log-base-foundation';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe responsável pelo gerenciamento de variável em memória/storage do dispositivo
 * Ao utilizar os métodos da classe para atualizar o valor em memória, por padrão,
 * o valor é refletido no storage
 * @template T
 */
class StorageUtil {
    /**
     * Construtor da classe
     * storage Recebe o Storage via Angular Dependency Injection
     * @param {?} storage
     */
    constructor(storage) {
        this.storage = storage;
        /**
         * Campo id para facilitar buscas
         */
        this.fieldId = '';
        /**
         * Conteúdo a ser refletido
         */
        this.storageData = {};
        /**
         * Indica que a classe já foi inicializada corretamente
         */
        this.initialized = false;
        /**
         * Se uma operação que trava o registro está em andamento
         */
        this.lock = false;
    }
    /**
     * @return {?}
     */
    get id() {
        return this.fieldId;
    }
    /**
     * @return {?}
     */
    get exclusive() {
        return this.exclusiveStorage;
    }
    /**
     * @return {?}
     */
    get suffixFn() {
        if (!this.suffixFunction) {
            return (/**
             * @return {?}
             */
            () => '');
        }
        else {
            return this.suffixFunction;
        }
    }
    /**
     * @return {?}
     */
    get initFn() {
        return this.initFunc;
    }
    /**
     * @return {?}
     */
    get name() {
        return this.storageName;
    }
    /**
     * @return {?}
     */
    get data() {
        if (!this.isInit) {
            throw new Error(`Storage: ${this.name} está sendo lido sem ser inicializado.`);
        }
        return this.getCurrentSuffixData();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set data(value) {
        if (!this.isInit) {
            throw new Error(`Storage: ${this.name} está sendo definido sem ser inicializado.`);
        }
        /** @type {?} */
        const suffix = this.currentSuffix;
        this.storageData[suffix] = value;
    }
    /**
     * Sufixo corrente quando storage exclusivo
     * @return {?}
     */
    get currentSuffix() {
        /** @type {?} */
        let suffix = this.suffixFn();
        if (!suffix || suffix === '') {
            suffix = StorageUtil.SUFFIX_DEFAULT;
        }
        return suffix;
    }
    /**
     * @return {?}
     */
    get isInit() {
        return this.initialized;
    }
    /**
     * @return {?}
     */
    get isLock() {
        return this.lock;
    }
    /**
     * Configura e realiza a leitura de dados do storage
     * { name:string, id? :string, exclusive?: boolean, suffixFn?: () => void, lazy?: boolean, initFn?: () => void}
     * @param {?=} __0
     * @return {?}
     */
    config({ name, id, exclusive, suffixFn, lazy, initFn, } = {
        name: '',
        id: '',
        exclusive: false,
        suffixFn: (/**
         * @return {?}
         */
        () => ''),
        lazy: false,
        initFn: (/**
         * @return {?}
         */
        () => undefined),
    }) {
        if (!name || name === '') {
            throw new Error('Storage necessita de um nome (name)');
        }
        if (exclusive && !suffixFn) {
            throw new Error('Storage exclusivo necessita de uma função para retornar o sufixo (suffixFn)');
        }
        this.storageName = name;
        this.fieldId = id;
        this.exclusiveStorage = exclusive;
        this.suffixFunction = suffixFn;
        this.initFunc = initFn;
        if (lazy) {
            return Promise.resolve(undefined);
        }
        else {
            return this.read();
        }
    }
    /**
     * Atualiza o storage para refletir o valor contido em memória
     * @return {?}
     */
    updateStorage() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isInit) {
                throw new Error('Necessário chamar algum dos métodos de leitura (get,read)');
            }
            yield UtilsFn.lock((/**
             * @return {?}
             */
            () => this.lock));
            this.lock = true;
            return yield this.storage
                .set(this.name, this.storageData)
                .then((/**
             * @return {?}
             */
            () => {
                this.lock = false;
                return this.data;
            }))
                .catch((/**
             * @param {?} err
             * @return {?}
             */
            err => {
                this.lock = false;
                console.error(err);
                return undefined;
            }));
        });
    }
    /**
     * Ferramenta utilizada para encontrar algum valor em um subnível
     * search
     * fields
     * \@example
     * storage.data = [{ nome: 'joao', endereco: {rua: 'qualquer', numero: 123}}]
     * storage.find(123,'endereco','numero').then(endereco =>{
     * endereco.numero = 234
     * storage.updateStorage()})
     * @param {?} search
     * @param {...?} fields
     * @return {?}
     */
    find(search, ...fields) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const array = this.data;
            array.find((/**
             * @param {?} element
             * @return {?}
             */
            element => {
                /** @type {?} */
                const ret = this.recursiveField(fields, element) === search;
                if (ret) {
                    fields.pop();
                    if (fields.length > 0) {
                        return this.recursiveField(fields, element);
                    }
                    else {
                        return element;
                    }
                }
                return ret;
            }));
            throw new Error('Item não encontrado no storage: ' + search);
        });
    }
    /**
     * Atualiza a variavel em memória e reflete no storage.
     * Pode-se acessar um subnível passando no fields o nível que deseja acessar,
     * \@example
     * // exemplo 1:
     * storage.data == {campoa:'valor', campob:{ campox: 'valor2'}}
     * storage.set('novovalor',true,'campob','campox')
     * // Resultado objeto: {campoa:'valor', campob:{ campox: 'novovalor'}}
     * // exemplo 2:
     * storage.data == [ {campoa:'valor', campob:{ campox: 'valor2'}}, {campoa:'valor', campob:{ campox: 'valork'}} ]
     * storage.set('novovalorx',true,1,'campob','campox')
     * // Caso atualizar o storage.data diretamente, chamar updateStorage para refletir os dados no storage
     * newData
     * save
     * fields
     * @param {?} newData
     * @param {?=} save
     * @param {...?} fields
     * @return {?}
     */
    set(newData, save = true, ...fields) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isInit) {
                throw new Error('Necessário chamar algum dos métodos de leitura (get,read)');
            }
            if (fields.length === 0) {
                this.data = newData;
            }
            else {
                /** @type {?} */
                const field = fields.pop();
                /** @type {?} */
                const data = this.recursiveField(fields, this.data);
                data[field] = newData;
            }
            if (save) {
                return yield this.updateStorage();
            }
            else {
                return this.data;
            }
        });
    }
    /**
     * Faz a leitura do storage se ainda não foi feita ou da variável de memória
     * search Busca em campo que representa uma chave primária do objeto
     * @param {?} search
     * @return {?}
     */
    get(search) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.read();
            if (this.id && search) {
                try {
                    /** @type {?} */
                    const array = this.data;
                    return array.find((/**
                     * @param {?} element
                     * @return {?}
                     */
                    element => {
                        return element[this.id] === search;
                    }));
                }
                catch (error) {
                    throw error;
                }
            }
            else {
                return this.data;
            }
        });
    }
    /**
     * Faz a leitura dos dados que estão no storage
     * @return {?}
     */
    read() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.name) {
                throw new Error('Necessário chamar o método config');
            }
            yield UtilsFn.lock((/**
             * @return {?}
             */
            () => this.lock));
            /** @type {?} */
            const suffix = this.currentSuffix;
            if (this.storageData.hasOwnProperty(suffix)) {
                return this.data;
            }
            else if (this.isInit) {
                this.storageData[suffix] = this.init(null);
            }
            else {
                return yield this.storage
                    .get(this.name)
                    .then((/**
                 * @param {?} value
                 * @return {?}
                 */
                value => {
                    return this.updateData(value);
                }))
                    .catch((/**
                 * @param {?} err
                 * @return {?}
                 */
                err => {
                    console.error(err);
                    return undefined;
                }));
            }
        });
    }
    /**
     * Remove um item de um array, atualiza no storage e retorna uma promessa de item atualizado.
     * search
     * @param {?} search
     * @return {?}
     */
    delete(search) {
        if (!this.isInit) {
            throw new Error('Necessário chamar algum dos métodos de leitura (get,read)');
        }
        if (this.id && search) {
            return new Promise((/**
             * @param {?} resolve
             * @param {?} reject
             * @return {?}
             */
            (resolve, reject) => {
                /** @type {?} */
                const array = this.data;
                /** @type {?} */
                const index = array.findIndex((/**
                 * @param {?} element
                 * @return {?}
                 */
                element => {
                    return element[this.id] === search;
                }));
                if (index > -1) {
                    array.splice(index, 1);
                    this.set(array).then((/**
                     * @param {?} v
                     * @return {?}
                     */
                    v => {
                        resolve(this.data);
                    }));
                }
                else {
                    reject(`Registro ${search} não encontrado.`);
                }
            }));
        }
    }
    /**
     * Atualiza o valor em memória lida do storage
     * value
     * @private
     * @param {?} value
     * @return {?}
     */
    updateData(value) {
        if (value === undefined || value === null || typeof value !== 'object') {
            this.storageData = {};
        }
        else {
            this.storageData = value;
        }
        this.initialized = true;
        return this.data;
    }
    /**
     * Inicializa o armazenamento com base na função que o usuário informou
     * @private
     * @param {?} data
     * @return {?}
     */
    init(data) {
        if ((data === undefined || data === null) && this.initFn) {
            data = this.initFn();
        }
        return data;
    }
    /**
     * Função recursíva para encontrar subníveis automáticamente
     * x array de campos recursivos
     * data a informação
     * level nível de iteração
     * @private
     * @param {?} fields
     * @param {?} data
     * @param {?=} level
     * @return {?}
     */
    recursiveField(fields, data, level = 0) {
        if (level < fields.length - 1) {
            return this.recursiveField(fields, data[fields[level]], ++level);
        }
        return data[fields[level]];
    }
    /**
     * Recupera os dados do sufíxo atual
     * @private
     * @return {?}
     */
    getCurrentSuffixData() {
        /** @type {?} */
        const suffix = this.currentSuffix;
        if (this.storageData.hasOwnProperty(suffix) === false) {
            this.storageData[suffix] = this.init(null);
        }
        return this.storageData[suffix];
    }
}
/**
 * Sufixo padrão
 */
StorageUtil.SUFFIX_DEFAULT = '__stDefault0__';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { StorageUtil };

//# sourceMappingURL=totvs-log-mobile-foundation-src-storage-util.js.map