import { Storage } from '@ionic/storage';
/**
 * Classe responsável pelo gerenciamento de variável em memória/storage do dispositivo
 * Ao utilizar os métodos da classe para atualizar o valor em memória, por padrão,
 * o valor é refletido no storage
 */
export declare class StorageUtil<T> {
    private storage;
    /** Sufixo padrão */
    private static SUFFIX_DEFAULT;
    /**
     * Campo id para facilitar buscas
     */
    private fieldId;
    readonly id: string;
    /**
     * Indica que o storage é exclusivo
     */
    private exclusiveStorage;
    readonly exclusive: boolean;
    /**
     * Indica a função de sufixo quando o storage é exclusivoIndica a função de sufixo quando o storage é exclusivo
     */
    private suffixFunction;
    readonly suffixFn: () => string;
    /**
     * Indica a função de inicialização de dados quando o storage está vazio
     */
    private initFunc;
    readonly initFn: () => T;
    /**
     * Nome da chave no storage para get/set
     */
    private storageName;
    readonly name: string;
    /**
     * Conteúdo a ser refletido
     */
    private storageData;
    data: T;
    /**
     * Sufixo corrente quando storage exclusivo
     */
    readonly currentSuffix: string;
    /**
     * Indica que a classe já foi inicializada corretamente
     */
    private initialized;
    readonly isInit: boolean;
    /**
     * Se uma operação que trava o registro está em andamento
     */
    private lock;
    readonly isLock: boolean;
    /**
     * Construtor da classe
     * storage Recebe o Storage via Angular Dependency Injection
     */
    constructor(storage: Storage);
    /**
     * Configura e realiza a leitura de dados do storage
     * { name:string, id? :string, exclusive?: boolean, suffixFn?: () => void, lazy?: boolean, initFn?: () => void}
     */
    config({ name, id, exclusive, suffixFn, lazy, initFn, }?: {
        name: string;
        id?: string;
        exclusive?: boolean;
        suffixFn?: () => string;
        lazy?: boolean;
        initFn?: () => T;
    }): Promise<void | T>;
    /**
     * Atualiza o storage para refletir o valor contido em memória
     */
    updateStorage(): Promise<T>;
    /**
     * Ferramenta utilizada para encontrar algum valor em um subnível
     * search
     * fields
     * @example
     * storage.data = [{ nome: 'joao', endereco: {rua: 'qualquer', numero: 123}}]
     * storage.find(123,'endereco','numero').then(endereco =>{
     * endereco.numero = 234
     * storage.updateStorage()})
     */
    find(search: any, ...fields: string[]): Promise<any>;
    /**
     * Atualiza a variavel em memória e reflete no storage.
     * Pode-se acessar um subnível passando no fields o nível que deseja acessar,
     * @example
     * // exemplo 1:
     * storage.data == {campoa:'valor', campob:{ campox: 'valor2'}}
     * storage.set('novovalor',true,'campob','campox')
     * // Resultado objeto: {campoa:'valor', campob:{ campox: 'novovalor'}}
     * // exemplo 2:
     * storage.data == [ {campoa:'valor', campob:{ campox: 'valor2'}}, {campoa:'valor', campob:{ campox: 'valork'}} ]
     * storage.set('novovalorx',true,1,'campob','campox')
     * // Caso atualizar o storage.data diretamente, chamar updateStorage para refletir os dados no storage
     * newData
     * save
     * fields
     */
    set(newData: any, save?: boolean, ...fields: string[]): Promise<T>;
    /**
     * Faz a leitura do storage se ainda não foi feita ou da variável de memória
     * search Busca em campo que representa uma chave primária do objeto
     */
    get(search: any): Promise<any>;
    /**
     * Faz a leitura dos dados que estão no storage
     */
    read(): Promise<void | T>;
    /**
     * Remove um item de um array, atualiza no storage e retorna uma promessa de item atualizado.
     * search
     */
    delete(search: any): Promise<T>;
    /**
     * Atualiza o valor em memória lida do storage
     * value
     */
    private updateData;
    /**
     * Inicializa o armazenamento com base na função que o usuário informou
     */
    private init;
    /**
     * Função recursíva para encontrar subníveis automáticamente
     * x array de campos recursivos
     * data a informação
     * level nível de iteração
     */
    private recursiveField;
    /**
     * Recupera os dados do sufíxo atual
     */
    private getCurrentSuffixData;
}
