import { InjectionToken, Injector } from '@angular/core';
import { NetworkTypes, ISyncWatch } from './sync-watch.interface';
import { Network } from '@ionic-native/network/ngx';
import { Storage } from '@ionic/storage';
import { Platform, Events } from '@ionic/angular';
import { ISyncEntity } from './sync-entity.interface';
import { ISyncConfig } from './sync-config.interface';
export declare const SYNC_CONFIG: InjectionToken<ISyncConfig>;
/**
 * Serviço provedor de sincronismo utilizando serviços externos
 */
export declare class SyncProvider {
    private storage;
    private config;
    private network;
    private events;
    private injector;
    /**
     * Constante convertendo 1 dia em milisegundos
     */
    static daysMs: number;
    /**
     * Constante convertendo 1 hora em milisegundos
     */
    static hoursMs: number;
    /**
     * Constante convertendo 1 minuto em milisegundos
     */
    static minutesMs: number;
    /**
     * Constante convertendo 1 segundo em milisegundos
     */
    static secondsMs: number;
    /**
     * @ignore
     */
    private syncing;
    /**
     * Se a sincronização está acontecendo
     */
    readonly isSyncing: boolean;
    /**
     * @ignore
     * Token do processamento temporizado de sincronismo
     */
    private intervalToken;
    /**
     * @ignore
     * Token do processamento quando executado sem agendamento
     */
    private nowToken;
    /**
     * @ignore
     * Token do processamento agendado do inicio do sincronizmo
     */
    private scheduleToken;
    /**
     * @ignore
     * Tipo da rede registrada
     */
    private currentNetworkType;
    /**
     * @ignore
     * Assinatura de mudanças de rede
     */
    private onNetworkChange;
    /**
     * Função que define a função para indicar o nome do storage
     */
    private storageNameFn;
    storageNameFunction: () => string;
    /**
     * Atributo que retorna o nome do storage
     */
    readonly storageName: string;
    /**
     * Retorna se o sincronismo está ativo
     */
    readonly autoWatchEnabled: boolean;
    /**
     * @ignore
     * Persistência dos dados do estado de sincronização
     */
    private syncStorage;
    /**
     * Construtor da classe
     * platform Platform do ionic
     * authService Serviço de autenticação
     * storage Serviço de armazenamento do ionic
     * config Token de configuração do serviço
     * network Serviço de rede do ionic
     * events Serviço de eventos do ionic
     * injector Injetor de dependências do angular
     */
    constructor(platform: Platform, storage: Storage, config: ISyncConfig, network: Network, events: Events, injector: Injector);
    /**
     * Dispara o sincronismo de todas as entidades registradas.
     * Consulte isSyncing para saber se a sincronização está em andamento.
     */
    syncAll(): Promise<void>;
    /**
     * Função recursiva que dispara a sincronização de entidades conforme ordem recebida
     * entities Lista com todas as entidades
     * order Ordem da execução
     */
    syncAllByOrder(entities: ISyncEntity[], order?: number): Promise<void>;
    /**
     * Dispara o sincronismo de todas as entidades registradas conforme ordem
     * Publica o evento SyncEvents.GOT_ENTITY_SYNC_DATA da finalização da sincronização da entidade.
     * entities Lista com as entidades que deverão ser sincronizadas
     * Indicação se ocorreu com sucesso
     */
    syncAllEntities(entities: ISyncEntity[]): Promise<boolean>;
    /**
     * Inicia o serviço que executa temporalmente a sincronização de todas as entidades,
     * caso o dispositivo esteja na rede escolhida
     * now Se o serviço deve iniciar imediatamente ou deve aguardar o próximo cíclo
     */
    startAutoWatch(now: boolean): void;
    /**
     * Cancela a sincronização automática.
     */
    stopAutoWatch(): void;
    /**
     * Permite que seja trocado quais os tipos de rede
     * que a sincronização automática está ativa
     * types
     */
    changeEnabledAutoWatchNetWorks(...types: NetworkTypes[]): void;
    /**
     * Permite que sejam trocadas as configurações de sincronização automática
     * newConfiguration Novas configurações
     */
    changeSyncWatch(newConfiguration?: ISyncWatch): void;
    /**
     * Recupera o estado de sincronização da entidade
     * entity a entidade de sincronização
     */
    private getState;
    /**
     * Recupera o índice do estado de sincronização da entidade no storage
     * entity a entidade de sincronização
     */
    private getStateIdx;
    /**
     * Cria um novo estado de sincronização para a entidade
     * entity a entidade de sincronização
     */
    private newState;
    /**
     * Calcula o tempo em ms para a próxima chamada da sincronização
     * Quando não configurado, o tempo será um minuto
     */
    private getNextCall;
    /**
     * @ignore
     * Configura os valores padrão, quando não informados
     */
    private setDefaults;
    /**
     * @ignore
     * Assina a todos os eventos do interesse da classe
     */
    private subscribeEvents;
    /**
     * Assina o evento de trocas de rede
     */
    private watchNetwork;
}
